<html>
<head>
	<title>Remailer Reliability Stats [<TMPL_VAR NAME="SITE_NAME">]</title>
	<META HTTP-EQUIV="Expires" CONTENT="<TMPL_VAR NAME="EXPIRES">">
	<link href="echolot.css" rel="stylesheet" type="text/css">
	<!-- Pinger-Version: Echolot <TMPL_VAR NAME="version">
	-->
	<!-- $Id: echolot.html 631 2004-04-28 11:58:25Z weasel $ -->
</head>
<body>
<h1>Remailer Reliability Stats [<TMPL_VAR NAME="SITE_NAME">]</h1>

<p>
This pinger automatically collects and generates remailer reliability statistics
for both Type I (Cypherpunk) and Type II (Mixmaster) remailers. It also
offers current keyrings <TMPL_IF NAME="thesaurus">and each remailer's reply to <code>remailer-xxx</code>
queries </TMPL_IF>for your convenience.

<h2>Available Stats</h2>

<p>
Out of the <TMPL_VAR NAME="CPUNK_TOTAL"> Cypherpunk and
<TMPL_VAR NAME="MIX_TOTAL"> Mixmaster remailers
(<TMPL_VAR NAME="UNIQUE_ADDRESSES"> unique addresses) on these stats there
are <TMPL_VAR NAME="CPUNK_98"> CPunks and <TMPL_VAR NAME="MIX_98"> Mixes
over 98.0% in terms of overall reliability.

<h3>HTML</h3>
<table border=1>
<tr>
  <th>Cypherpunk (Type I)</th> 
  <th>Mixmaster (Type II)</th>
<TMPL_IF NAME="combined_list">
	<th>Combined</th>
</TMPL_IF>
</tr>
<tr>
  <td><a href="rlist.html"><code>rlist.html</code></a>
<TMPL_IF NAME="separate_rlist">
     <SMALL>
     (<a href="rlist-rsa.html">rsa</a>,
      <a href="rlist-dsa.html">dsa</a>,
      <a href="rlist-clear.html">unencrypted</a>)
			</SMALL>
</TMPL_IF>
  </td>
  <td><a href="mlist.html"><code>mlist.html</code></a></td>
<TMPL_IF NAME="combined_list">
	<td rowspan=2><a href="clist.html"><code>clist2.html</code></a></td>
</TMPL_IF>
</tr>
<tr>
  <td><a href="rlist2.html"><code>rlist2.html</code></a>
<TMPL_IF NAME="separate_rlist">
     <SMALL>
		 (<a href="rlist2-rsa.html">rsa</a>,
      <a href="rlist2-dsa.html">dsa</a>,
      <a href="rlist2-clear.html">unencrypted</a>)
			</SMALL>
</TMPL_IF>
  </td>
  <td><a href="mlist2.html"><code>mlist2.html</code></a></td>
</tr>
</table>

<h3>Plain Text</h3>
<table border=1>
<tr>
  <th>Cypherpunk (Type I)</th> 
  <th>Mixmaster (Type II)</th>
<TMPL_IF NAME="combined_list">
	<th>Combined</th>
</TMPL_IF>
</tr>
<tr>
  <td><a href="rlist.txt"><code>rlist.txt</code></a>
<TMPL_IF NAME="separate_rlist">
     <SMALL>
     (<a href="rlist-rsa.txt">rsa</a>,
      <a href="rlist-dsa.txt">dsa</a>,
      <a href="rlist-clear.txt">unencrypted</a>)
			</SMALL>
</TMPL_IF>
  </td>
  <td><a href="mlist.txt"><code>mlist.txt</code></a></td>
<TMPL_IF NAME="combined_list">
	<td rowspan=2><a href="clist.txt"><code>clist2.txt</code></a></td>
</TMPL_IF>
</tr>
<tr>
  <td><a href="rlist2.txt"><code>rlist2.txt</code></a>
<TMPL_IF NAME="separate_rlist">
     <SMALL>
		 (<a href="rlist2-rsa.txt">rsa</a>,
      <a href="rlist2-dsa.txt">dsa</a>,
      <a href="rlist2-clear.txt">unencrypted</a>)
			</SMALL>
</TMPL_IF>
  </td>
  <td><a href="mlist2.txt"><code>mlist2.txt</code></a></td>
</tr>
</table>


<h2>Available Keyrings</h2>
<table border=1>
	<tr>
		<th>Cypherpunk (Type I)</th> 
		<th>Mixmaster (Type II)</th>
	</tr>
	<tr>
		<td><a href="pgp-rsa.asc">RSA only</a></td>
		<td><a href="pubring.mix"><code>pubring.mix</code></a></td>
	</tr>
	<tr>
		<td><a href="pgp-all.asc">RSA and DSA</a></td>
		<td><a href="type2.list"><code>type2.list</code></a></td>
	</tr>
</table>

<TMPL_IF NAME="thesaurus">
<h2>Thesaurus</h2>

Find the Thesaurus index in <a href="thesaurus/">thesaurus/</a>.
</TMPL_IF>

<TMPL_IF NAME="fromlines">
<h2>From Header Lines</h2>

A summary of <a href="from.html">From Headers</a> each remailer uses has also
been compiled.  It also lists which remailers allow setting the From: Header.
</TMPL_IF>

<hr>
Created by <a href="http://www.palfrader.org/echolot/">Echolot <TMPL_VAR
NAME="version"></a> at <TMPL_VAR NAME="CURRENT_TIMESTAMP"> (individual pages
may be newer).<br>
This pinger is operated by <code><TMPL_VAR NAME="operator"></code>.<br>
All times in Coordinated Universal Time (UTC).
</body>

<!-- vim:set ts=2: -->
<!-- vim:set shiftwidth=2: -->
</html>
