<html>
<head>
	<title>From Headers [<TMPL_VAR NAME="SITE_NAME">]</title>
	<META HTTP-EQUIV="Expires" CONTENT="<TMPL_VAR NAME="EXPIRES">">
	<link href="echolot.css" rel="stylesheet" type="text/css">
	<!-- $Id: fromlinesindex.html 631 2004-04-28 11:58:25Z weasel $ -->
</head>
<body>
<h1>From Headers [<TMPL_VAR NAME="SITE_NAME">]</h1>

<p>
<a href="./">Up</a>

<h2>Default From: Headers</h2>

<p>
The default From headers of all remailers when sending e-mail.  Middleman
remailers are not listed here.

<p>
<table border=1>
<tr><tr><th>nick</th><th>Disclaimer</th><th>From Line</th><th>type</th></tr>

<TMPL_LOOP NAME="default">
	<TMPL_LOOP NAME="data">
		<tr>
			<td><TMPL_VAR NAME="nick"></td>
			<td align="center"><TMPL_IF NAME="disclaim_top">top</TMPL_IF>&nbsp;<TMPL_IF NAME="disclaim_bot">bot</TMPL_IF></td>
			<td><TMPL_VAR NAME="from"></td>
			<td><small><TMPL_VAR NAME="types"></small></td>
		</tr>
	</TMPL_LOOP>
</TMPL_LOOP>

</table>

<h2>User Supplied From: Headers</h2>

<p>
If a message sent with a user supplied From header differs from
one without, the remailer is listed here.  The difference can either
be an additional disclaimer, actually allowing (partial) From headers,
etc.

<p>
<table border=1>

<tr><tr><th>nick</th><th>Disclaimer</th><th>From Line</th><th>type</th></tr>

<TMPL_LOOP NAME="usersupplied">
	<TMPL_LOOP NAME="data">
		<tr>
			<td><TMPL_VAR NAME="nick"></td>
			<td align="center"><TMPL_IF NAME="disclaim_top">top</TMPL_IF>&nbsp;<TMPL_IF NAME="disclaim_bot">bot</TMPL_IF></td>
			<td><TMPL_VAR NAME="from"></td>
			<td><small><TMPL_VAR NAME="types"></small></td>
		</tr>
	</TMPL_LOOP>
</TMPL_LOOP>

</table>

<hr>
Created by <a href="http://www.palfrader.org/echolot/">Echolot <TMPL_VAR NAME="version"></a>.<br>
Last update: <TMPL_VAR NAME="CURRENT_TIMESTAMP">.
</body>

<!-- vim:set ts=2: -->
<!-- vim:set shiftwidth=2: -->
</html>
