body {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none}
h1 { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 21px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
h2 { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 19px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
h3 { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 17px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
.tablehead { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: bolder; font-variant: normal; text-transform: none ; text-decoration: none}
td { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
ul { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
li { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
tr { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
a { font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 15px; font-style: normal; line-height: normal; font-weight: normal; font-variant: normal; text-transform: none }
