<H2><a name="legend">Legend</a></H2>
<!-- $Id: v1legend.html 631 2004-04-28 11:58:25Z weasel $ -->
<p>
<strong>History</strong>: The result of test messages sent in the last 12 days.
<p>
<BLOCKQUOTE>
<TABLE CELLPADDING=0 CELLSPACING=0 border="0">
<TR><TD>?</TD><TD>No test message sent</TD></TR>
<TR><TD>(space)</TD><TD>Response not received</TD></TR>
<TR><TD>#</TD><TD>Response in less than 5 minutes</TD></TR>
<TR><TD>*</TD><TD>Response in less than 1 hour</TD></TR>
<TR><TD>+</TD><TD>Response in less than 4 hours</TD></TR>
<TR><TD>-</TD><TD>Response in less than 24 hours</TD></TR>
<TR><TD>.</TD><TD>Response in less than 2 days</TD></TR>
<TR><TD>_</TD><TD>Response in more than 2 days</TD></TR>
</TABLE>
</BLOCKQUOTE>
<p>
<strong>Latency</strong>: The average response time of the remailer. 
<p>
<strong>Uptime</strong>: The fraction of responses received from tests sent in
the last 12 days.
